#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 ~ 2014 Deepin, Inc.
#               2011 ~ 2014 Kaisheng Ye
# 
# Author:     Kaisheng Ye <kaisheng.ye@gmail.com>
# Maintainer: Kaisheng Ye <kaisheng.ye@gmail.com>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import xappy
import sqlite3
from deepin_utils.file import create_directory, get_parent_dir
from deepin_utils.config import Config


def get_update_data_dir():
    DATA_DIR = os.path.join(get_parent_dir(__file__), "data")
    data_newest_id_path = os.path.join(DATA_DIR, "data_newest_id.ini")
    newest_data_id_config = Config(data_newest_id_path)
    newest_data_id_config.load()
    newest_data_id = newest_data_id_config.get("newest", "data_id")
    return os.path.join(get_parent_dir(__file__, 1), "data", "update", newest_data_id)

class SoftwareDB:
    def __init__(self):
        self.db_path = os.path.join(get_update_data_dir(), "software", "software.db")

    def init_connect(self):
        self.connect = sqlite3.connect(self.db_path)
        self.cursor = self.connect.cursor()

class DesktopDB:
    def __init__(self):
        self.db_path = os.path.join(get_update_data_dir(), "desktop", "desktop2014.db")

    def init_connect(self):
        self.connect = sqlite3.connect(self.db_path)
        self.cursor = self.connect.cursor()

def build_index(remove_old=True, language_code='zh_CN'):
    '''
    init docs
    '''
    search_db_path = os.path.join(get_update_data_dir(), "search", language_code, "search_db")
    if remove_old:
        create_directory(search_db_path, True)
    
    # Init. indexer connection.
    iconn = xappy.IndexerConnection(search_db_path)
    
    # Add pkg_name field for exact package name from search results.
    iconn.add_field_action("pkg_name", xappy.FieldActions.STORE_CONTENT) 
    
    # Add desktop_file field to sort search results.
    iconn.add_field_action("have_desktop_file", xappy.FieldActions.SORTABLE, type='float')
    
    # Add keyword_term field to index keyword term for fuzzy search.
    iconn.add_field_action("keyword_term", 
                           xappy.FieldActions.INDEX_FREETEXT, 
                           language='en', 
                           nopos=True # don't store postion information (huge size)
                           )
    
    # Connect software db.
    software_db = SoftwareDB()
    software_db.init_connect()

    # Connect desktop db.
    desktop_db = DesktopDB()
    desktop_db.init_connect()
    
    # Index package information in xapian database.
    software_db.cursor.execute(
            "SELECT id FROM language WHERE language_code=?", [language_code])
    language_id = software_db.cursor.fetchone()[0]
    software_db.cursor.execute(
        "SELECT pkg_name, alias_name, short_desc, long_desc FROM software WHERE language_id=?", [language_id])
    import jieba
    for (pkg_name, alias_name, short_desc, long_desc) in software_db.cursor.fetchall():
        pkg_doc = xappy.UnprocessedDocument()
        
        pkg_doc.fields.append(xappy.Field("pkg_name", pkg_name))
        
        desktop_db.cursor.execute(
            "SELECT * FROM package WHERE pkg_name=?", [pkg_name])
        if desktop_db.cursor.fetchone() == None:
            pkg_doc.fields.append(xappy.Field("have_desktop_file", 1))
        else:
            pkg_doc.fields.append(xappy.Field("have_desktop_file", 0))
        
        terms = list(set(list(jieba.cut(pkg_name)) + list(jieba.cut(alias_name)) + list(jieba.cut(long_desc)) + list(jieba.cut(short_desc))))
        pkg_doc.fields.append(xappy.Field("keyword_term", ' '.join(terms)))
        
        iconn.add(pkg_doc)
        
    # Close indexer connection.
    iconn.close()    

def search_query(keywords):
    '''
    init docs
    '''
    # Init search connect.
    search_db_path = os.path.join(get_update_data_dir(), "search", "zh_CN", "search_db")
    sconn = xappy.SearchConnection(search_db_path)
    
    # Do search.
    search = ' '.join(keywords)
    q = sconn.query_parse(search, default_op=sconn.OP_AND)
    results = sconn.search(q, 0, sconn.get_doccount(), sortby="have_desktop_file")
    
    return map(lambda result: result.data["pkg_name"][0], results)
        
if __name__ == "__main__":
    build_index()
